_header(`C')
<P>
For compilation, _man(`tcc(1)'), _man(`gcc(1)'), and _man(`clang(1)') are all reasonable choices depending on your needs.
</P>
<P>
On Alpine Linux, the standard library headers are in the _code(`musl-dev') package.
</P>
<P>
In most system package managers, standard library documentation can be found in the _code(`man-pages') and _code(`man-pages-posix') packages.
</P>
<P>
Many Linux software distributions' system package managers have meta-packages that pull all necessities for C development as dependencies.
Alpine has _code(`build-base') and Debian has _code(`build-essential').
</P>
<P>
For linking to libraries, see _ref(`#pkg-config').
</P>

_subheader(`Prototypical C (1978)')
<UL>
<LI><A HREF="https://archive.org/details/TheCProgrammingLanguageFirstEdition">The C Programming Language, 1st ed.</A> (The Internet Archive)</LI>
<LI><A HREF="https://www.bell-labs.com/usr/dmr/www/cman.pdf">C Reference Manual</A> (this paper bears striking similarity to and was also written by the author of Appendix A of <I>The C Programming Language, 1st ed.</I> - the chronology of these writings' release is unknown)</LI>
<LI><A HREF="https://arstechnica.com/features/2020/12/a-damn-stupid-thing-to-do-the-origins-of-c/">"A damn stupid thing to do" - the origins of C</A></LI>
<LI><A HREF="https://www.bell-labs.com/usr/dmr/www/chist.html">The Development of the C Language</A></LI>
<LI><A HREF="http://pdp11.aiju.de/">PDP-11 Emulator</A></LI>
</UL>
<P>
The C programming language was first described to the public in <I>The C Programming Language</I> by Brian W. Kernighan and Dennis M. Ritchie, published by Prentice Hall in 1978.
The language evolved out of the former B programming language, which was a product of Ken Thompson:
</P>
<FIGURE>
<BLOCKQUOTE>
<P>Challenged by McIlroy's feat in reproducing TMG, Thompson decided that Unix—possibly it had not even been named yet—needed a system programming language. After a rapidly scuttled attempt at Fortran, he created instead a language of his own, which he called B. B can be thought of as C without types; more accurately, it is BCPL squeezed into 8K bytes of memory and filtered through Thompson's brain. Its name most probably represents a contraction of BCPL, though an alternate theory holds that it derives from Bon [Thompson 69], an unrelated language created by Thompson during the Multics days. Bon in turn was named either after his wife Bonnie, or (according to an encyclopedia quotation in its manual), after a religion whose rituals involve the murmuring of magic formulas.</P>
</BLOCKQUOTE>
<FIGCAPTION>(<CITE>The Development of the C Language</CITE>)</FIGCAPTION>
</FIGURE>
<P>[Thompson 69] references <CODE>K. Thompson, `Bon—an Interactive Language,' undated AT&amp;T Bell Laboratories internal memorandum (ca. 1969)</CODE>. This is possibly <A HREF="http://people.csail.mit.edu/saltzer/Multics/MHP-Saltzer-060508/filedrawers/180.btl-misc/Scan%204.PDF">Bon User's Manual</A>.
Please get in touch if you have a hyperlink for this document as I can't find it indexed on any search engines.</P>
<P>
This language described in <I>The C Programming Language</I> isn't the C programming language known by most.
It was a pre-standardization, relatively prototypical C, and rather than being codified in ANSI its primary documentation was the book by Kernighan and Ritchie (this book would later be known colloquially as <I>K&amp;R</I>).
This was known as "pre-ANSI C" or "K&amp;R C" and dubbed "C78" for the purposes of naming this page, in the same style as the later "C89", described in ANSI X3.159-1989, or "C11", described in ISO/IEC 9899:2011.
"C78" was also the name given to historical C by the <CODE>c78(7)</CODE> manual page on FreeBSD 9.0.
</P>
<P>
"C78" is incompatible with "C89" and later standards.
This page documents those incompatibilities and relative oddities.
</P>
<P>
I have never done (and probably never will do) extensive programming in pre-ANSI C.
These incompatibilities were discovered out of Appendix C in <I>The C Programming Language, 2nd ed.</I> but are described further.
</P>
<H4 ID="octal89"><CODE>8</CODE> and <CODE>9</CODE> as valid octal digits</H4>
<UL>
<LI><A HREF="https://minnie.tuhs.org/cgi-bin/utree.pl?file=V6/usr/source/c">UNIX V6 C compiler source directory</A></LI>
</UL>
<PRE>
main()
{
	printf("%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n",
		00, 01, 02, 03, 04, 05, 06, 07, 08, 09
	);
}
</PRE>
<P>
On my machine, the GNU C compiler emits the following errors (this omits warnings) for the preceding piece of code:
</P>
<PRE>
snippet.c:4:49: error: invalid digit "8" in octal constant
    4 |                 00, 01, 02, 03, 04, 05, 06, 07, 08, 09
      |                                                 ^~
snippet.c:4:53: error: invalid digit "9" in octal constant
    4 |                 00, 01, 02, 03, 04, 05, 06, 07, 08, 09
      |                                                     ^~
</PRE>
<P>
This is because in C (both pre-standardization and post-ANSI) integer constants with leading zeroes are parsed as octal (base 8) numbers.
In pre-ANSI C, <CODE>8</CODE> and <CODE>9</CODE> were valid octal digits corresponding to <CODE>010</CODE>(b8) and <CODE>011</CODE>(b8).
This is documented in <I>The C Programming Language</I>, Appendix A, subsection 2.4.1, and evidenced by the preceding code block.
The following is output of the compiled program in UNIX V6 on an emulated PDP-11:
</P>
<PRE>
0
1
2
3
4
5
6
7
8
9
</PRE>
<P>
This behavior exists within <CODE>/usr/source/c/c0t.s</CODE>, which is a PDP-11 assembler file that parses integer constants (as far as I can tell).
This file provides <CODE>getnum</CODE>, a function used in <CODE>/usr/source/c/c00.c</CODE>.
My theory is that this behavior is a side-effect of a very efficient but imperfect method of parsing integer constants from a file stream.
</P>
<P>
This wouldn't be a significant or even noticeable error; most programmers wouldn't use <CODE>8</CODE> or <CODE>9</CODE> as octal digits anyway.
If not for its documentation within <I>The C Programming Language</I> it would probably be an obscure bug.
This is also mentioned in <I>The C Programming Language, 2nd ed.</I> in Appendix C.
</P>
_subheader(`C1989')
_subheader(`C1999')
<UL>
<LI>_link(`C99', `https://en.wikipedia.org/wiki/C99') (Wikipedia)</LI>
<LI>_link(`c99(1p)', `https://www.man7.org/linux/man-pages/man1/c99.1p.html')</LI>
</UL>
_subheader(`C2011')
_subheader(`C2017')
_subheader(`C202x')
